@@ -11,7 +11,7 @@ import operator
 import itertools
 from functools import partial
 
-from .struct import struct, Struct, transpose_structs, map_type, traverse_type, reduce_type
+from .struct import map_tree, struct, Struct, transpose_structs, map_type, traverse_type, reduce_type
 
 
 class Table(Struct):
@@ -187,6 +187,14 @@ class Histogram:
 def map_arrays(data, f, *args, **kwargs):
     return map_type(data,  np.ndarray, partial(f, *args, **kwargs))  
 
+def map_array_like(data, f, *args, **kwargs):
+  def g(x):
+    if hasattr(x, 'shape'):
+      return f(x)
+    else:
+      return None
+  return map_tree(data, g, *args, **kwargs)      
+
 def traverse_arrays(data, f, *args, **kwargs):
     return traverse_type(data,  np.ndarray, partial(f, *args, **kwargs))  
 
@@ -196,10 +204,10 @@ def reduce_arrays(data, f, op, initializer=None):
 
 
 def shape_info(x):
-    return map_arrays(x, lambda x: tuple([*x.shape, type(x), x.dtype]))
+    return map_array_like(x, lambda x: tuple([*x.shape, x.dtype]))
 
 def shape(x):
-    return map_arrays(x, lambda x: tuple(x.shape))
+    return map_array_like(x, lambda x: tuple(x.shape))
 
 def arrays_astype(t, **kwargs):
     return map_arrays(t, np.ndarray.astype, **kwargs)        
